@@ -3,7 +3,7 @@
 //! Print some information about a Blender scene file.
 //! ```shell
 //! $ ./target/release/blend_info -h
-//! blend_info 0.2.7
+//! blend_info 0.2.8
 //! Print some information about a Blender scene file
 //!
 //! USAGE:
@@ -25,13 +25,13 @@
 //! ## Usage as a crate (used in your own code)
 //!
 //! ```rust
-//! use blend_info::{read_dna, DnaStrC};
+//! use blend_info::{print_pointer, read_dna, use_dna, DnaStrC};
 //! // std
 //! use std::collections::HashMap;
 //! use std::path::PathBuf;
-//!
+//! 
 //! fn main() -> std::io::Result<()> {
-//!     println!("read_dna");
+//!     println!("use_dna");
 //!     // .blend file
 //!     let path: PathBuf = PathBuf::from(r"blend/factory_v279.blend");
 //!     println!("Try to read {:?} file ...", path);
@@ -43,7 +43,7 @@
 //!     let mut types: Vec<String> = Vec::new();
 //!     let mut num_bytes_read: usize = 0;
 //!     let print_dna: bool = false;
-//!     let print_pointers: bool = false;
+//!     let print_pointers: bool = true;
 //!     read_dna(
 //!         print_dna,
 //!         print_pointers,
@@ -55,7 +55,39 @@
 //!         &mut types,
 //!         &mut num_bytes_read,
 //!     )?;
-//!     println!("{} {:?}", num_bytes_read, &path);
+//!     println!("{} bytes read by read_dna() ...", num_bytes_read);
+//!     // use DNA
+//!     println!(
+//!         "Try to read {:?} file again (camera/lamp centric this time) ...",
+//!         path
+//!     );
+//!     let print_dna: bool = true;
+//!     let names: Vec<String> = vec!["Camera".to_string(), "Lamp".to_string()];
+//!     let mut bytes_read: Vec<u8> = Vec::with_capacity(num_bytes_read);
+//!     let mut structs_read: Vec<String> = Vec::with_capacity(names.len());
+//!     let mut data_read: Vec<u32> = Vec::with_capacity(names.len());
+//!     let mut pointers_read: Vec<(usize, u32)> = Vec::with_capacity(names.len());
+//!     use_dna(
+//!         print_dna,
+//!         &path,
+//!         &dna_types_hm,
+//!         &dna_structs_hm,
+//!         &names,
+//!         &dna_2_type_id,
+//!         &types,
+//!         &mut bytes_read,
+//!         &mut structs_read,
+//!         &mut data_read,
+//!         &mut pointers_read,
+//!     )?;
+//!     println!("bytes_read: {:?}", bytes_read);
+//!     println!("structs_read: {:?}", structs_read);
+//!     println!("data_read: {:?}", data_read);
+//!     println!("pointers_read: {:?}", pointers_read);
+//!     print!("SDNAnr = {}: ", pointers_read[0].1);
+//!     print_pointer(pointers_read[0].0, &names[0], &dna_pointers_hm);
+//!     print!("SDNAnr = {}: ", pointers_read[1].1);
+//!     print_pointer(pointers_read[1].0, &names[1], &dna_pointers_hm);
 //!     Ok(())
 //! }
 //! ```
@@ -227,9 +259,12 @@ use std::fs::File;
 use std::io::Read;
 use std::mem;
 
+/// Each Blender C struct can have member entries, with a type and a name.
 #[derive(Debug, Clone)]
 pub struct DnaStrMember {
+    /// Type of struct member, e.g. "float"
     pub mem_type: String,
+    /// Name of struct member, e.g. "lens"
     pub mem_name: String,
 }
 
@@ -242,9 +277,43 @@ impl DnaStrMember {
     }
 }
 
+/// Each Blender versions stores a number of DnaStrC blocks, which
+/// represent C structs in the original C code.
+///
+/// e.g.
+/// ```shell
+/// $ ./target/release/blend_info --dna blend/factory_v279.blend | grep " Camera (" -B 1 -A 23
+///   [SDNAnr = 25]
+///   Camera (len=248) {
+///     ID id;
+///     AnimData *adt;
+///     char type;
+///     char dtx;
+///     short flag;
+///     float passepartalpha;
+///     float clipsta;
+///     float clipend;
+///     float lens;
+///     float ortho_scale;
+///     float drawsize;
+///     float sensor_x;
+///     float sensor_y;
+///     float shiftx;
+///     float shifty;
+///     float YF_dofdist;
+///     Ipo *ipo;
+///     Object *dof_ob;
+///     GPUDOFSettings gpu_dof;
+///     char sensor_fit;
+///     char pad[7];
+///     CameraStereoSettings stereo;
+///   }
+/// ```
 #[derive(Debug)]
 pub struct DnaStrC {
+    /// SDNAnr: ID within Blender for struct names
     pub sdna_nr: u32,
+    /// A vector of struct members (type and name)
     pub members: Vec<DnaStrMember>,
 }
 
@@ -467,6 +536,48 @@ fn read_type_names(
     Ok(())
 }
 
+/// Each Blender C struct can have member entries and this function
+/// calculates a size for each of these entries.
+///
+/// e.g.
+/// ```shell
+/// $ ./target/release/blend_info -n Camera blend/factory_v279.blend
+/// Camera 248
+/// struct Camera { // SDNAnr = 25
+///   ID id; // 120
+///   AnimData *adt; // 8
+///   char type; // 1
+///   char dtx; // 1
+///   short flag; // 2
+///   float passepartalpha; // 4
+///   float clipsta; // 4
+///   float clipend; // 4
+///   float lens; // 4
+///   float ortho_scale; // 4
+///   float drawsize; // 4
+///   float sensor_x; // 4
+///   float sensor_y; // 4
+///   float shiftx; // 4
+///   float shifty; // 4
+///   float YF_dofdist; // 4
+///   Ipo *ipo; // 8
+///   Object *dof_ob; // 8
+///   GPUDOFSettings gpu_dof; // 24
+///   char sensor_fit; // 1
+///   char pad[7]; // 7
+///   CameraStereoSettings stereo; // 24
+/// }; // 248
+/// ```
+///
+/// Simple members, like "char", "short", or "float" consist e.g. of
+/// one, two, or 4 bytes, whereas ID is a C struct itself (and the
+/// number of bytes might change over time between Blender versions).
+/// Pointers to a C struct start with an "*" in it's name
+/// (e.g. "*ipo"), and always have the same length. independent of the
+/// length of the C struct they are pointing to. Arrays (like
+/// "pad\[7\]") multiply the length of the array (in this case 7) by the
+/// number of bytes used for the type (in this case one byte for a
+/// "char").
 pub fn calc_mem_tlen(member: &DnaStrMember, type_found: u16) -> u16 {
     let mut mem_tlen: u16 = 0;
     // check first char for '*' (pointer)
@@ -564,6 +675,7 @@ pub fn calc_mem_tlen(member: &DnaStrMember, type_found: u16) -> u16 {
     mem_tlen
 }
 
+/// Try to extract a String from a C struct with a "\*name\*" member.
 pub fn get_id_name(
     member: &DnaStrMember,
     bytes_read: &[u8],
@@ -597,6 +709,7 @@ pub fn get_id_name(
     return_str
 }
 
+/// Extract byte by index, if member type matches "char", otherwise return 0_u8.
 pub fn get_char(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> u8 {
     let mut char_value: u8 = 0;
     if member.mem_type.as_str() == "char" {
@@ -605,6 +718,9 @@ pub fn get_char(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) ->
     char_value
 }
 
+/// Extract 4 bytes in a row and interpret those as a "float".
+///
+/// Print a WARNING in case the member type is **not** a "float" (and return 0.0_f32).
 pub fn get_float(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> f32 {
     let mut float_value: f32 = 0.0;
     if member.mem_type.as_str() == "float" {
@@ -619,6 +735,9 @@ pub fn get_float(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) ->
     float_value
 }
 
+/// Try to extract an array of 2 floats.
+///
+/// Print a WARNING in case the member type is **not** a "float" (and return array of two 0.0_f32).
 pub fn get_float2(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 2] {
     let mut float_values: [f32; 2] = [0.0; 2];
     if member.mem_type.as_str() == "float" {
@@ -635,6 +754,9 @@ pub fn get_float2(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     float_values
 }
 
+/// Try to extract an array of 3 floats.
+///
+/// Print a WARNING in case the member type is **not** a "float" (and return array of three 0.0_f32).
 pub fn get_float3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 3] {
     let mut float_values: [f32; 3] = [0.0; 3];
     if member.mem_type.as_str() == "float" {
@@ -651,6 +773,9 @@ pub fn get_float3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     float_values
 }
 
+/// Try to extract an array of 4 floats.
+///
+/// Print a WARNING in case the member type is **not** a "float" (and return array of four 0.0_f32).
 pub fn get_float4(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 4] {
     let mut float_values: [f32; 4] = [0.0; 4];
     if member.mem_type.as_str() == "float" {
@@ -667,6 +792,9 @@ pub fn get_float4(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     float_values
 }
 
+/// Extract 4 bytes in a row and interpret those as a "int".
+///
+/// Print a WARNING in case the member type is **not** an "int" (and return 0_i32).
 pub fn get_int(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i32 {
     let mut int_value: i32 = 0;
     if member.mem_type.as_str() == "int" {
@@ -680,6 +808,9 @@ pub fn get_int(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i
     int_value
 }
 
+/// Try to extract 16 "float" values and return them as an array of f32 values.
+///
+/// Print a WARNING in case the member type is **not** a "float" (and return array of 16 0.0_f32).
 pub fn get_matrix(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [f32; 16] {
     let mut mat_values: [f32; 16] = [0.0_f32; 16];
     if member.mem_type.as_str() == "float" {
@@ -695,10 +826,13 @@ pub fn get_matrix(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
                 skip_bytes += 4;
             }
         }
+    } else {
+        println!("WARNING: \"float\" expected, {:?} found", member.mem_type);
     }
     mat_values
 }
 
+/// Try to extract 8 bytes in a row and interpret those as a pointer (in memory).
 pub fn get_pointer(_member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> usize {
     let mut pointer_value: usize = 0;
     pointer_value += (bytes_read[byte_index] as usize) << 0;
@@ -712,6 +846,9 @@ pub fn get_pointer(_member: &DnaStrMember, bytes_read: &[u8], byte_index: usize)
     pointer_value
 }
 
+/// Extract 2 bytes in a row and interpret those as a "short".
+///
+/// Print a WARNING in case the member type is **not** an "short" (and return 0_i16).
 pub fn get_short(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> i16 {
     let mut short_value: i16 = 0;
     if member.mem_type.as_str() == "short" {
@@ -723,6 +860,9 @@ pub fn get_short(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) ->
     short_value
 }
 
+/// Try to extract an array of 3 shorts.
+///
+/// Print a WARNING in case the member type is **not** a "short" (and return array of three 0_i16).
 pub fn get_short3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -> [i16; 3] {
     let mut short_values: [i16; 3] = [0; 3];
     if member.mem_type.as_str() == "short" {
@@ -738,6 +878,15 @@ pub fn get_short3(member: &DnaStrMember, bytes_read: &[u8], byte_index: usize) -
     short_values
 }
 
+/// Print information about a pointer (byte position in binary .blend file vs. memory address).
+///
+/// e.g.
+/// ```shell
+/// $ ./target/release/blend_info -n Object.*data blend/factory_v279.blend
+/// Object.*data = 0x000567dc (0x00007fa32df90a08)
+/// Object.*data = 0x00056d3c (0x00007fa31ef6b608)
+/// Object.*data = 0x000568ec (0x00007fa3334c6608)
+/// ```
 pub fn print_pointer(
     pointer: usize,
     struct_name: &String,
@@ -755,7 +904,7 @@ pub fn print_pointer(
     }
 }
 
-/// Read a `.blend` file to extract DNA information first
+/// Read a `.blend` file to extract DNA information first.
 ///
 /// Verbosity flags:
 ///
@@ -1138,6 +1287,33 @@ pub fn read_dna(
     Ok(())
 }
 
+/// Use the DNA types and structs returned by read_dna().
+///
+/// Read the .blend file (again), but limit the returned data by
+/// filtering out information provided as input by the names vector of
+/// strings.
+///
+/// Verbosity flags:
+///
+/// * `print_dna`
+///
+/// Input
+///
+/// * `path` - a path to a folder/filename.blend
+/// * `names` - filter out by list of interesting things
+///
+/// Input returned by previous read_dna(...) call
+///
+/// * `dna_types_hm` - A HashMap with a type name (e.g. "float") and it's byte size as u16
+/// * `dna_structs_hm` - A HashMap with a struct name (e.g. "Camera") and it's members (DnaStrC)
+/// * `dna_2_type_id` - Use `sdna_nr` to find `type_id`
+/// * `types` - Vec of type names (e.g. "char", "short", "double")
+///
+/// Return values (`&mut`):
+/// * `bytes_read` - All bytes read which relate to input names
+/// * `structs_read` - All struct names read as a list in order of bytes read
+/// * `data_read` - List of chunk sizes (same length as *structs_read*)
+/// * `pointers_read` - List of tuples (pointer as usize and SDNAnr)
 pub fn use_dna(
     print_dna: bool,
     path: &std::path::PathBuf,
